<?php
/**
 * Utility classes to use Pashua when building GUIs for CLI scripts
 *
 * License: PHP License 3.0 (http://www.php.net/license/3_0.txt)
 * Copyright: Jesus M. Castagnetto, 2004
 *
 * @package Pashua
 * @author Jesus M. Castagnetto <jmcastagnetto@php.net>
 * @version 0.1.0
 */

require_once 'PEAR.php';

class PashuaAttribute {/*{{{*/
    var $_name;
    var $_value;
    var $_parent;

    function PashuaAttribute($name, $value) {
        $this->setName($name);
        $this->setValue($value);
    }
    
    function setName($name) {
        $this->_name = $name;
    }

    function setValue($value) {
        $this->_value = $value;
    }

    function setParent(&$parent) {
        $this->_parent = &$parent;
    }

    function toString() {
        return $this->_parent->getName()."_{$this->_name}={$this->_value}";
    }
}/*}}}*/

class PashuaBasicElement {/*{{{*/
    var $_name;
    var $_attrs;

    function PashuaBasicElement($name, $type) {
         $this->_attrs = array();
         $this->_name = $name;
         $this->addAttribute('type', $type);
    }

    function getName() {
        return $this->_name;
    }

    function addAttribute($name, $value) {
        $att = &new PashuaAttribute($name, $value);
        $att->setParent(&$this);
        $this->_attrs[] = &$att;
    }

    function setLabel($value) {
        $this->addAttribute('label', $value);
    }

    function setTooltip($value) {
        $this->addAttribute('tooltip', $value);
    }

    function toString() {
        $out = '';
        foreach ($this->_attrs as $att) {
            $out .= $att->toString()."\n";
        }
        return $out;
    }
}/*}}}*/

class PashuaDefaultButton extends PashuaBasicElement {/*{{{*/
    function PashuaDefaultButton($name) {
        $this->PashuaBasicElement($name, 'defaultbutton');
    }
}/*}}}*/

class PashuaCancelButton extends PashuaBasicElement {/*{{{*/
    function PashuaCancelButton($name) {
        $this->PashuaBasicElement($name, 'defaultbutton');
    }
}/*}}}*/

class PashuaButton extends PashuaBasicElement {/*{{{*/
    function PashuaButton($name) {
        $this->PashuaBasicElement($name, 'defaultbutton');
    }
}/*}}}*/

class PashuaTextfield extends PashuaBasicElement {/*{{{*/

    function PashuaTextField($name) {
        $this->PashuaBasicElement($name, 'textfield');
    }

    function setWidth($width) {
        $this->addAttribute('width', $width);
    }

    function setDefault($default) {
        $this->addAttribute('default', $default);
    }
}/*}}}*/

class PashuaPopup extends PashuaBasicElement {/*{{{*/

    function PashuaPopup($name) {
        $this->PashuaBasicElement($name, 'popup');
    }

    function addOption($option) {
        $this->addAttribute('option', $option);
    }

    function setDefault($default) {
        $this->addAttribute('default', $default);
    }
}/*}}}*/

class PashuaCombobox extends PashuaBasicElement {/*{{{*/

    function PashuaCombobox($name) {
        $this->PashuaBasicElement($name, 'combobox');
    }

    function setWidth($with) {
        $this->addAttribute('width', $width);
    }

    function addOption($Option) {
        $this->addAttribute('option', $option);
    }

    function setDefault($default) {
        $this->addAttribute('default', $default);
    }
}/*}}}*/

class PashuaRadiobutton extends PashuaBasicElement {/*{{{*/

    function PashuaRadiobutton($name) {
        $this->PashuaBasicElement($name, 'radiobutton');
    }

    function addOption($option) {
        $this->addAttribute('option', $option);
    }

    function setDefault($default) {
        $this->addAttribute('default', $default);
    }
}/*}}}*/

class PashuaOpenbrowser extends PashuaBasicElement {/*{{{*/

    function PashuaOpenbrowser($name) {
        $this->PashuaBasicElement($name, 'openbrowser');
    }

    function setWidth($width) {
        $this->addAttribute('width', $width);
    }

    function setOption($option) {
        $this->addAttribute('option', $option);
    }

    function setFiletype($filetype) {
        $this->addAttribute('filetype', $filetype);
    }

    function setDefault($default) {
        $this->addAttribute('default', $default);
    }
}/*}}}*/

class PashuaSavebrowser extends PashuaBasicElement {/*{{{*/

    function PashuaSavebrowser($name) {
        $this->PashuaBasicElement($name, 'savebrowser');
    }

    function setWidth($width) {
        $this->addAttribute('width', $width);
    }

    function setOption($option) {
        $this->addAttribute('option', $option);
    }

    function setFiletype($filetype) {
        $this->addAttribute('filetype', $filetype);
    }

    function setDefault($default) {
        $this->addAttribute('default', $default);
    }
}/*}}}*/

class PashuaText extends PashuaBasicElement {/*{{{*/

    function PashuaText($name) {
        $this->PashuaBasicElement($name, 'text');
    }

    function setWidth($width) {
        $this->addAttribute('width', $width);
    }

    function setText($text) {
        $this->addAttribute('text', $text);
    }
}/*}}}*/

class PashuaTextBox extends PashuaBasicElement {/*{{{*/

    function PashuaText($name) {
        $this->PashuaBasicElement($name, 'textbox');
    }

    function setWidth($width) {
        $this->addAttribute('width', $width);
    }

    function setHeight($height) {
        $this->addAttribute('height', $height);
    }

    function setFontsizeRegular() {
        $this->addAttribute('fontsize', 'regular');
    }

    function setFontsizeSmall() {
        $this->addAttribute('fontsize', 'small');
    }

    function setFontsizeMini() {
        $this->addAttribute('fontsize', 'mini');
    }

    function setDefault($default) {
        $this->addAttribute('default', $default);
    }
}/*}}}*/

class PashuaCheckbox extends PashuaBasicElement {/*{{{*/

    function PashuaCheckbox($name) {
        $this->PashuaBasicElement($name, 'checkbox');
    }

    function setDefault($default) {
        $this->addAttribute('default', $default);
    }
}/*}}}*/

class PashuaPassword extends PashuaBasicElement {/*{{{*/

    function PashuaPassword($name) {
        $this->PashuaBasicElement($name, 'password');
    }

    function setWidth($width) {
        $this->addAttribute('width', $width);
    }

    function setDefault($default) {
        $this->addAttribute('default', $default);
    }
}/*}}}*/

class PashuaImage extends PashuaBasicElement {/*{{{*/

    function PashuaImage($name) {
        $this->PashuaBasicElement($name, 'image');
    }

    function setPath($path) {
        $this->addAttribute('path', $path);
    }

    function setMaxWidth($maxwidth) {
        $this->addAttribute('maxwidth', $maxwidth);
    }

    function setMaxHeight($maxheight) {
        $this->addAttribute('maxheight', $maxheight);
    }

    function setBorderOn() {
        $this->addAttribute('border', 1);
    }

    function setBorderOff() {
        $this->addAttribute('border', 0);
    }

}/*}}}*/

class PashuaSpecialElement {/*{{{*/
    var $_name;
    var $_value;
    
    function PashuaSpecialElement($name, $value) {
        $this->_name = $name;
        $this->_value = $value;
    }

    function toString() {
        return "{$this->_name}={$this->_value}\n";
    }
}/*}}}*/

class PashuaSeparator {/*{{{*/
    function toString() {
        return "-\n";
    }
}/*}}}*/

class PashuaConfiguration {/*{{{*/
    var $_elements;

    function PashuaConfiguration() {
        $this->_elements = array();
    }

    function setWindowtitle($title) {
        $this->_elements[] = &new PashuaSpecialElement('windowtitle', $title);
    }

    function setTransparency($transparency) {
        $this->_elements[] = &new PashuaSpecialElement('transparency', $transparency);
    }

    function setAppearance($appearance) {
        $this->_elements[] = &new PashuaSpecialElement('appearance', $appearance);
    }
    
    function addElement(&$elem) {
        $this->_elements[] = &$elem;
    }

    function addSeparator() {
        $this->_elements[] = &new PashuaSeparator();
    }

    function toString() {
        $out = '';
        foreach ($this->_elements as $elem) {
            $out .= $elem->toString();
        }
        return $out;
    }
}/*}}}*/

class Pashua {/*{{{*/

    var $_pashuaBinary = null;
    var $_config;

    function Pashua($pashuaBinary='') {
        if (!empty($pashuaBinary)) {
            $this->setInterpreter($pashuaBinary);
        }
    }

    function setInterpreter($pashuaBinary) {
        $this->_pashuaBinary = $pashuaBinary;
    }

    function _guessInterpreter() {
        // Try to figure out the path to pashua
        $bundlepath = "Pashua.app/Contents/MacOS/Pashua";
        $paths = array(
                    '/Applications/Pashua/'.$bundlepath,
                    dirname(__FILE__).'/Pashua', 
                    dirname(__FILE__)."/$bundlepath",
                    "./$bundlepath", 
                    '/Applications/$bundlepath',
                    "$_SERVER[HOME]/Applications/$bundlepath");
        foreach ($paths as $path) {
            if (file_exists($path) and is_executable($path))
                // Looks like Pashua is in $dir >> we can exit the loop
                break;
        }

        return empty($path) ? null : $path;
    }

    function getInterpreter() {
        if (is_null($this->_pashuaBinary)) {
            $this->_pashuaBinary = $this->_guessInterpreter();
        }
        return $this->_pashuaBinary;
    }

    function setConfiguration (&$config) {
        $this->_config = $config;
    }

    function &getConfiguration() {
        return $this->_config;
    }

    function run() {

        if (is_null($this->getInterpreter())) {
            return PEAR::raiseError('Could not find the Pashua executable');
        }

        if (is_null($this->getConfiguration())) {
            return PEAR::raiseError('Configuration has not been set');
        }

        $fname = tempnam('/tmp', 'Pashua_');
        $fp = fopen($fname, 'w');
        fwrite($fp, $this->_config->toString());
        fclose($fp);

        $cmd = "'{$this->_pashuaBinary}' $fname";
        $result = `$cmd`;

        //echo "*** $fname \n\n";
        unlink($fname);

        // Init result array for E_ALL compatibility
        $parsed = array();

        // Parse result
        foreach (explode("\n", $result) as $line) {
            preg_match('/^(\w+)=(.*)$/', $line, $matches);
            if (empty($matches) || empty($matches[1])) {
                continue;
            }
            $parsed[trim($matches[1])] = trim($matches[2]);
        }

        return $parsed;
    }

}/*}}}*/

// vim: set fdm=marker:
?>
